import argparse
import logging
import os

from nymbus.cli.runner import Runner
from nymbus.cli.setupper import Setupper

logger = logging.getLogger(__name__)


def main():

    # Nymbus
    parser = argparse.ArgumentParser()
    parser.add_argument("--debug", help="Print a stacktrace of errors", action="store_true")
    parser.set_defaults(action=lambda args: parser.parse_args(["-h"]))
    subparsers = parser.add_subparsers()

    # Setup
    setup = subparsers.add_parser("setup")
    setup.set_defaults(action=lambda args, rest: Setupper().manage(args.component))

    # Run
    run = subparsers.add_parser("run")
    run.add_argument("component", help="Path to the folder containing the <env>.yml files (relative or not)")
    run.add_argument("environment", help="Environment name (of the <env>.yml target file)")
    run.add_argument("--context", default=None, help="Path to the folder that will be visibile in docker steps")
    run.add_argument("--step", default=None, help="Step name (in the <env>.yml file)")
    run.set_defaults(action=lambda args, rest: Runner().run(args.component, args.environment, args.step, args.context))

    # Parse args
    arguments, remainder = parser.parse_known_args()

    try:

        # Invoke the corresponding functions
        arguments.action(arguments, remainder)
        logger.info("All done.")

    except Exception as e:

        if arguments.debug:
            logger.exception("")
        else:
            logger.error(e if str(e) else f"Error: {type(e)}")
        exit(1)


if __name__ == '__main__':
    main()



from nymbus.config.images.bind import Bind


class Image:

    def __init__(self, yml: dict):
        # Pop the config entries
        config = yml.copy()
        self.name = config.pop("name")
        self.volumes = {
            path: Bind(bind)
            for path, bind in config.pop("volumes", {}).items()
        }




from nymbus.config.environment import expand_variable


class EnvSpec:

    def __init__(self, yml: dict):
        # Pop the config entries
        self.env = yml.get("env", {})

    def expand(self):
        self.env = {name: expand_variable(value) for name, value in self.env.items()}

    @classmethod
    def from_env(cls, env: dict):
        return cls({"env": env})



import logging
from pathlib import Path
from typing import Dict

from nymbus.config.environment import DEFAULT_NYMBUS_EXTENSION
from nymbus.config.envspec import EnvSpec
from nymbus.config.readers.reader import read_yml
from nymbus.config.components.step import Step

logger = logging.getLogger(__name__)


class Component(EnvSpec):

    def __init__(self, name: str, environment: str, yml: dict):
        super().__init__(yml)
        self.name = name
        self.environment = environment

        # Pop the config entries
        config = yml.copy()
        self.context = config.pop("context", None)
        self.steps: Dict[str, Step] = {
            step: Step(step, environment, step_config)
            for step, step_config in config.pop("steps", {}).items()
        }

        # If there are still configs, they are unknown. Print a warning (for retro-compatibility)
        config.pop("env", {})
        if config:
            logger.warning(f"Unknown configuration in component \"{self.name}\", "
                           f"file {self.environment}{DEFAULT_NYMBUS_EXTENSION}: {config}")

    @classmethod
    def from_file(cls, location: Path):
        return cls(location.parent.name, read_yml(location))



import logging
from pathlib import Path

import docker
from git import Repo

from nymbus.config.environment import NYMBUS_OUTPUT_FOLDER
from nymbus.config.envspec import EnvSpec
from nymbus.config.components.step import Step
from nymbus.tools.shell import Shell


logger = logging.getLogger(__name__)

DOCKER_CONTEXT = '/mnt/.nymbus/context'
DOCKER_OUTPUT = '/mnt/.nymbus/component'
DOCKER_TEMPLATE = '/mnt/.nymbus/template'


def run(step: Step, env_spec: EnvSpec, component_path: Path, context: str):

    # Touch the folder
    nymbus_folder = component_path / NYMBUS_OUTPUT_FOLDER
    nymbus_folder.mkdir(parents=True, exist_ok=True)

    # Does it have a template?
    template = None
    if step.template:

        # Clone the repo if not existing
        repo_dir = nymbus_folder / "templates" / step.name / step.template.tag
        if not repo_dir.exists():
            logger.info(f"Cloning {step.template.repository}...")
            Repo.clone_from(
                step.template.repository,
                repo_dir,
                branch=step.template.tag
            )
        template = repo_dir

    # Does it have an associated image?
    if not step.image:
        run_locally(step, env_spec, component_path, context, template=template)
    else:
        run_in_docker(step, env_spec, component_path, context, template=template)


def run_locally(step: Step, env_spec: EnvSpec, component_path: Path, context: str, template: Path = None):
    # Run it locally
    Shell().run(
        step.command,
        cwd=context,
        env=env_spec.env
    )


def run_in_docker(step: Step, env_spec: EnvSpec, component_path: Path, context: str, template: Path = None):

    volumes = {
        str(component_path/NYMBUS_OUTPUT_FOLDER): {'bind': '/mnt/.nymbus/component', 'mode': 'rw'},
        Path(context).absolute(): {'bind': '/mnt/.nymbus/context', 'mode': 'ro'},
        **{path: vars(bind) for path, bind in step.image.volumes.items()},
    }
    volumes = volumes if not template else {**volumes, str(template): {'bind': '/mnt/.nymbus/template', 'mode': 'ro'}}

    # Pull the image
    client = docker.from_env()
    logger.info(f"Pulling '{step.image.name}'...")
    client.images.pull(step.image.name)

    # Run in Docker container
    logger.info(f"Running '{step.command}'...")
    logger.info("-"*60)
    logs = client.containers.run(
        step.image.name,
        step.command if step.command else f"{DOCKER_TEMPLATE}/nymbus.sh",
        environment={
            **env_spec.env,
            "NYMBUS_CONTEXT": DOCKER_CONTEXT,
            "NYMBUS_OUTPUT": DOCKER_OUTPUT,
            "NYMBUS_TEMPLATE": DOCKER_TEMPLATE
        },
        volumes=volumes,
        remove=True,
        stderr=True,
        stream=True
    )
    for log in logs:
        logger.info(log.strip().decode())
    logger.info("-" * 60)

